"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getVersionMessage = exports.DeployCommand = void 0;
const fs = require("fs");
const source_deploy_retrieve_1 = require("@salesforce/source-deploy-retrieve");
const core_1 = require("@salesforce/core");
const ts_types_1 = require("@salesforce/ts-types");
const kit_1 = require("@salesforce/kit");
const sourceCommand_1 = require("./sourceCommand");
class DeployCommand extends sourceCommand_1.SourceCommand {
    constructor() {
        super(...arguments);
        this.displayDeployId = (0, kit_1.once)((id) => {
            if (!this.isJsonOutput()) {
                this.ux.log(`Deploy ID: ${id}`);
            }
        });
        // used to determine the correct stash.json key
        this.isSourceStash = true;
    }
    /**
     * Request a report of an in-progress or completed deployment.
     *
     * @param id the Deploy ID of a deployment request
     * @returns DeployResult
     */
    async report(id) {
        const deployId = this.resolveDeployId(id);
        this.displayDeployId(deployId);
        const res = await this.org.getConnection().metadata.checkDeployStatus(deployId, true);
        const deployStatus = res;
        const componentSet = this.componentSet || new source_deploy_retrieve_1.ComponentSet();
        return new source_deploy_retrieve_1.DeployResult(deployStatus, componentSet);
    }
    setStash(deployId) {
        const file = this.getStash();
        this.logger.debug(`Stashing deploy ID: ${deployId} in ${file.getPath()}`);
        file.writeSync({
            [this.getStashKey()]: { jobid: deployId },
        });
    }
    /**
     * This method is here to provide a workaround to stubbing a constructor in the tests.
     *
     * @param id
     */
    createDeploy(id) {
        return new source_deploy_retrieve_1.MetadataApiDeploy({ usernameOrConnection: this.org.getUsername(), id });
    }
    getStashKey() {
        return this.isSourceStash ? DeployCommand.SOURCE_STASH_KEY : DeployCommand.MDAPI_STASH_KEY;
    }
    resolveDeployId(id) {
        let stash;
        if (id) {
            return id;
        }
        else {
            try {
                stash = this.getStash();
                stash.readSync(true);
                const deployId = stash.get(this.getStashKey()).jobid;
                this.logger.debug(`Using deploy ID: ${deployId} from ${stash.getPath()}`);
                return deployId;
            }
            catch (err) {
                const error = err;
                if (error.name === 'JsonParseError') {
                    const stashFilePath = stash === null || stash === void 0 ? void 0 : stash.getPath();
                    const corruptFilePath = `${stashFilePath}_corrupted_${Date.now()}`;
                    fs.renameSync(stashFilePath, corruptFilePath);
                    const invalidStashErr = core_1.SfdxError.create('@salesforce/plugin-source', 'deploy', 'InvalidStashFile', [
                        corruptFilePath,
                    ]);
                    invalidStashErr.message = `${invalidStashErr.message}\n${error.message}`;
                    invalidStashErr.stack = `${invalidStashErr.stack}\nDue to:\n${error.stack}`;
                    throw invalidStashErr;
                }
                if (error.code === 'ENOENT' || !(stash === null || stash === void 0 ? void 0 : stash.get(this.getStashKey()))) {
                    // if the file doesn't exist, or the key doesn't exist in the stash
                    throw core_1.SfdxError.create('@salesforce/plugin-source', 'deploy', 'MissingDeployId');
                }
                throw core_1.SfdxError.wrap(error);
            }
        }
    }
    // REST is the default unless:
    //   1. SOAP is specified with the soapdeploy flag on the command
    //   2. The restDeploy SFDX config setting is explicitly false.
    async isRestDeploy() {
        if ((0, ts_types_1.getBoolean)(this.flags, 'soapdeploy') === true) {
            this.logger.debug('soapdeploy flag === true.  Using SOAP');
            return false;
        }
        const aggregator = await core_1.ConfigAggregator.create();
        const restDeployConfig = aggregator.getPropertyValue('restDeploy');
        // aggregator property values are returned as strings
        if (restDeployConfig === 'false') {
            this.logger.debug('restDeploy SFDX config === false.  Using SOAP');
            return false;
        }
        else if (restDeployConfig === 'true') {
            this.logger.debug('restDeploy SFDX config === true.  Using REST');
            return true;
        }
        else {
            this.logger.debug('soapdeploy flag unset. restDeploy SFDX config unset.  Defaulting to SOAP');
        }
        return false;
    }
    async poll(deployId, options) {
        var _a, _b;
        const defaultOptions = {
            frequency: (_a = options === null || options === void 0 ? void 0 : options.frequency) !== null && _a !== void 0 ? _a : kit_1.Duration.seconds(1),
            timeout: (_b = options === null || options === void 0 ? void 0 : options.timeout) !== null && _b !== void 0 ? _b : this.flags.wait,
            poll: async () => {
                const deployResult = await this.report(deployId);
                return {
                    completed: (0, ts_types_1.getBoolean)(deployResult, 'response.done'),
                    payload: deployResult,
                };
            },
        };
        const pollingOptions = { ...defaultOptions, ...options };
        const pollingClient = await core_1.PollingClient.create(pollingOptions);
        return pollingClient.subscribe();
    }
    getStash() {
        return new core_1.ConfigFile({ isGlobal: true, filename: 'stash.json' });
    }
}
exports.DeployCommand = DeployCommand;
DeployCommand.SOURCE_STASH_KEY = 'SOURCE_DEPLOY';
DeployCommand.MDAPI_STASH_KEY = 'MDAPI_DEPLOY';
const getVersionMessage = (action, componentSet, isRest) => {
    var _a;
    // commands pass in the.componentSet, which may not exist in some tests
    if (!componentSet) {
        return;
    }
    // neither
    if (!componentSet.sourceApiVersion && !componentSet.apiVersion) {
        return `*** ${action} with ${isRest ? 'REST' : 'SOAP'} ***`;
    }
    // either OR both match (SDR will use either)
    if (!componentSet.sourceApiVersion ||
        !componentSet.apiVersion ||
        componentSet.sourceApiVersion === componentSet.apiVersion) {
        return `*** ${action} with ${isRest ? 'REST' : 'SOAP'} API v${(_a = componentSet.apiVersion) !== null && _a !== void 0 ? _a : componentSet.sourceApiVersion} ***`;
    }
    // has both but they don't match
    return `*** ${action} v${componentSet.sourceApiVersion} metadata with ${isRest ? 'REST' : 'SOAP'} API v${componentSet.apiVersion} connection ***`;
};
exports.getVersionMessage = getVersionMessage;
//# sourceMappingURL=deployCommand.js.map